@@ -1,4 +1,5 @@
 // Copyright (C) 2018 The Qt Company Ltd.
+// Copyright (C) 2023 David Edmundson <davidedmundson@kde.org>
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "mockcompositor.h"
@@ -26,11 +27,12 @@ private slots:
     void switchPopups();
     void hidePopupParent();
     void pongs();
-    void minMaxSize_data();
     void minMaxSize();
     void windowGeometry();
     void foreignSurface();
     void nativeResources();
+    void suspended();
+    void initiallySuspended();
 };
 
 void tst_xdgshell::init()
@@ -607,47 +609,12 @@ void tst_xdgshell::pongs()
     QCOMPARE(pongSpy.first().at(0).toUInt(), serial);
 }
 
-void tst_xdgshell::minMaxSize_data()
-{
-    QTest::addColumn<QSize>("initialMinSize");
-    QTest::addColumn<QSize>("initialMaxSize");
-    QTest::addColumn<QSize>("nextMinSize");
-    QTest::addColumn<QSize>("nextMaxSize");
-    QTest::addColumn<QSize>("expectedInitialMinSize");
-    QTest::addColumn<QSize>("expectedInitialMaxSize");
-    QTest::addColumn<QSize>("expectedNextMinSize");
-    QTest::addColumn<QSize>("expectedNextMaxSize");
-
-    QTest::newRow("onlyMinSize") << QSize(50, 60) << QSize() << QSize(500, 600) << QSize()
-                                 << QSize(50, 60) << QSize(0, 0) << QSize(500, 600) << QSize(0, 0);
-
-    QTest::newRow("onlyMaxSize") << QSize() << QSize(70, 80) << QSize() << QSize(700, 800)
-                                 << QSize(0,0 ) << QSize(70, 80) << QSize(0, 0) << QSize(700, 800);
-
-    QTest::newRow("maxIsSentAsZero") << QSize() << QSize(QWINDOWSIZE_MAX, QWINDOWSIZE_MAX) << QSize() << QSize()
-                                 << QSize(0,0 ) << QSize(0, 0) << QSize(0, 0) << QSize(0, 0);
-
-
-    QTest::newRow("fullHints") << QSize(50, 60) << QSize(700, 800) << QSize(500, 600) << QSize(710, 810)
-                               << QSize(50, 60) << QSize(700, 800) << QSize(500, 600) << QSize(710, 810);
-
-    // setting a minimum above the maximum is not allowed, we should no-op
-    QTest::newRow("invalidResize") << QSize(50, 60) << QSize(100, 100) << QSize(500, 600) << QSize(100, 100)
-                                   << QSize(50, 60) << QSize(100, 100) << QSize(50, 60) << QSize(100, 100);}
-
 void tst_xdgshell::minMaxSize()
 {
-    QFETCH(QSize, initialMinSize);
-    QFETCH(QSize, initialMaxSize);
-
-    QFETCH(QSize, expectedInitialMinSize);
-    QFETCH(QSize, expectedInitialMaxSize);
-
     QRasterWindow window;
-    if (initialMinSize.isValid())
-        window.setMinimumSize(initialMinSize);
-    if (initialMaxSize.isValid())
-        window.setMaximumSize(initialMaxSize);
+    window.setMinimumSize(QSize(100, 100));
+    window.setMaximumSize(QSize(1000, 1000));
+    window.resize(400, 320);
     window.show();
     QCOMPOSITOR_TRY_VERIFY(xdgToplevel());
 
@@ -655,21 +622,16 @@ void tst_xdgshell::minMaxSize()
 
     // we don't roundtrip with our configuration the initial commit should be correct
 
-    QCOMPOSITOR_TRY_COMPARE(xdgToplevel()->m_committed.minSize, expectedInitialMinSize);
-    QCOMPOSITOR_TRY_COMPARE(xdgToplevel()->m_committed.maxSize, expectedInitialMaxSize);
+    QCOMPOSITOR_TRY_COMPARE(xdgToplevel()->m_committed.minSize, QSize(100, 100));
+    QCOMPOSITOR_TRY_COMPARE(xdgToplevel()->m_committed.maxSize, QSize(1000, 1000));
 
-    QFETCH(QSize, nextMinSize);
-    QFETCH(QSize, expectedNextMinSize);
-    window.setMinimumSize(nextMinSize);
+    window.setMaximumSize(QSize(500, 400));
     window.update();
-    QCOMPOSITOR_TRY_COMPARE(xdgToplevel()->m_committed.minSize, expectedNextMinSize);
+    QCOMPOSITOR_TRY_COMPARE(xdgToplevel()->m_committed.maxSize, QSize(500, 400));
 
-    QFETCH(QSize, nextMaxSize);
-    QFETCH(QSize, expectedNextMaxSize);
-
-    window.setMaximumSize(nextMaxSize);
+    window.setMinimumSize(QSize(50, 40));
     window.update();
-    QCOMPOSITOR_TRY_COMPARE(xdgToplevel()->m_committed.maxSize, expectedNextMaxSize);
+    QCOMPOSITOR_TRY_COMPARE(xdgToplevel()->m_committed.minSize, QSize(50, 40));
 }
 
 void tst_xdgshell::windowGeometry()
@@ -736,5 +698,35 @@ void tst_xdgshell::nativeResources()
     QCOMPARE(xdg_popup_proxy, nullptr);
 }
 
+void tst_xdgshell::suspended()
+{
+    QRasterWindow window;
+    window.resize(400, 320);
+    window.show();
+    QVERIFY(!window.isExposed()); // not exposed until we're configured
+    QCOMPOSITOR_TRY_VERIFY(xdgToplevel());
+
+    exec([=] { xdgToplevel()->sendCompleteConfigure(); });
+    QCOMPOSITOR_TRY_VERIFY(xdgToplevel()->m_xdgSurface->m_committedConfigureSerial);
+    QTRY_VERIFY(window.isExposed());
+
+    exec([=] { xdgToplevel()->sendCompleteConfigure(QSize(), {XdgToplevel::state_suspended}); });
+    QTRY_VERIFY(!window.isExposed());
+
+    exec([=] { xdgToplevel()->sendCompleteConfigure(QSize(), {}); });
+    QTRY_VERIFY(window.isExposed());
+}
+
+void tst_xdgshell::initiallySuspended()
+{
+    QRasterWindow window;
+    window.resize(400, 320);
+    window.show();
+    QVERIFY(!window.isExposed());
+    QCOMPOSITOR_TRY_VERIFY(xdgToplevel());
+    exec([=] { xdgToplevel()->sendCompleteConfigure(QSize(), {XdgToplevel::state_suspended}); });
+    QVERIFY(!window.isExposed());
+}
+
 QCOMPOSITOR_TEST_MAIN(tst_xdgshell)
 #include "tst_xdgshell.moc"
